import 'dart:io';
import 'dart:typed_data';
import 'package:flutter/material.dart';
import 'package:modern_form/modern_form_core.dart';
import 'package:modern_form/modern_form_files.dart';
import 'package:rxdart/rxdart.dart';
import 'package:youtube_explode_dart/youtube_explode_dart.dart';

class YoutubeHelper {
  final YoutubeHttpClient? youtubeHttpClient;

  YoutubeHelper([this.youtubeHttpClient]);

  Future<Uint8List?> downloadFromYoutube(String url,
      [void Function(int, int)? onBytesReceived, bool? force]) async {
    String filePath = await ModernFormFileHelper.getFilePathFromUrl(url, "mp4");
    File file = File(filePath);
    try {
      ModernFormProgressDialog? progressDialog;

      if (onBytesReceived == null) {
        progressDialog = ModernFormProgressDialog();
        progressDialog.showProgressDialog(
          textToBeDisplayed: 'Fazendo download do vídeo...',
          progressColor: Colors.white,
          barrierColor: Color(0x88222222),
        );
      }

      bool exists = await file.exists();
      bool isForce = force != null && force;
      if (isForce && exists) {
        await file.delete();
        file = await file.create();
      }

      if (!exists || isForce) {
        YoutubeExplode yt = factoryYoutubeExplode();
        MuxedStreamInfo streamInfo = await getStreamInfo(yt, url);

        Stream<List<int>> ytStream =
            yt.videos.streamsClient.get(streamInfo).asBroadcastStream();

        final BehaviorSubject<List<int>> controller =
            BehaviorSubject<List<int>>();

        if (onBytesReceived != null) {
          controller.addStream(ytStream);

          controller.listen((event) {
            onBytesReceived(streamInfo.size.totalBytes, file.lengthSync());
          });
        }

        IOSink fileStream = file.openWrite();

        await ytStream.pipe(fileStream);

        await fileStream.flush();
        await fileStream.close();

        controller.close();
        yt.close();
      }

      Uint8List data = await file.readAsBytes();

      if (progressDialog != null) {
        await progressDialog.dismissProgressDialog();
      }

      return data;
    } catch (e) {
      try {
        await file.delete();
      } catch (e) {
        print("ERROR downloadVideoFromYoutube file.delete() --> $e");
      }
      print("ERROR downloadVideoFromYoutube: $url --> $e");
      return null;
    }
  }

  @visibleForTesting
  Future<MuxedStreamInfo> getStreamInfo(
    YoutubeExplode yt,
    String url,
  ) async {
    StreamManifest manifest = await yt.videos.streamsClient.getManifest(url);

    int length = manifest.muxed.length;
    MuxedStreamInfo streamInfo = length > 1
        ? manifest.muxed.toList()[length - 1]
        : manifest.muxed.toList()[0];

    return streamInfo;
  }

  Future<int?> getYoutubeDownLoadLength(String url) async {
    try {
      YoutubeExplode yt = factoryYoutubeExplode();
      MuxedStreamInfo streamInfo = await getStreamInfo(yt, url);
      int length = streamInfo.size.totalBytes;

      yt.close();

      return length;
    } catch (e, s) {
      print("ERROR getYoutubeDownLoadLength(): $e --> $s");
      return -1;
    }
  }

  YoutubeExplode factoryYoutubeExplode() => YoutubeExplode(youtubeHttpClient);
}
